/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2017 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

describe('ModalGUISettings', function () {
  const _localStorage = {
    setItem: sinon.stub(),
    removeItem: sinon.stub(),
    getItem: sinon.stub().returns(false)
  };

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('modal-gui-settings', function () {
      done();
    });
  });

  function visibleRender(hide = sinon.stub(), handleChange = sinon.stub(), state = {
    'disable-cookie': false,
    'force-containers': false,
    'disable-auto-place': false
  }, handleSave = sinon.stub()) {
    return React.createElement(
      'div',
      { className: 'modal modal--narrow' },
      React.createElement(
        'div',
        { className: 'twelve-col no-margin-bottom' },
        React.createElement(
          'h2',
          { className: 'bordered' },
          'Custom GUI Settings'
        ),
        React.createElement(
          'span',
          { className: 'close', tabIndex: '0', role: 'button',
            onClick: hide },
          React.createElement(juju.components.SvgIcon, { name: 'close_16',
            size: '16' })
        )
      ),
      React.createElement(
        'div',
        { className: 'content' },
        React.createElement(
          'p',
          null,
          React.createElement(
            'label',
            { htmlFor: 'disable-cookie' },
            React.createElement('input', { type: 'checkbox', name: 'disable-cookie',
              id: 'disable-cookie',
              onChange: handleChange,
              defaultChecked: state['disable-cookie'] }),
            '\xA0 Disable the EU cookie warning.'
          )
        ),
        React.createElement(
          'p',
          null,
          React.createElement(
            'label',
            { htmlFor: 'force-containers' },
            React.createElement('input', { type: 'checkbox', name: 'force-containers',
              id: 'force-containers',
              onChange: handleChange,
              defaultChecked: state['force-containers'] }),
            '\xA0 Enable container control for this provider.'
          )
        ),
        React.createElement(
          'p',
          null,
          React.createElement(
            'label',
            { htmlFor: 'disable-auto-place' },
            React.createElement('input', { type: 'checkbox', name: 'disable-auto-place',
              id: 'disable-auto-place',
              onChange: handleChange,
              defaultChecked: state['disable-auto-place'] }),
            '\xA0 Default to not automatically place units on commit.'
          )
        ),
        React.createElement(
          'p',
          null,
          React.createElement(
            'small',
            null,
            'NOTE: You will need to reload for changes to take effect.'
          )
        ),
        React.createElement('input', { type: 'button', className: 'button--positive',
          name: 'save-settings', onClick: handleSave,
          id: 'save-settings', value: 'Save' })
      )
    );
  }

  it('renders', function () {
    const close = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.ModalGUISettings, {
      closeModal: close,
      localStorage: _localStorage }), true);
    let output = renderer.getRenderOutput();
    let expected = visibleRender(close);
    expect(output).toEqualJSX(expected);
  });

  it('saves state', function () {
    const close = sinon.stub();
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.ModalGUISettings, {
      closeModal: close,
      localStorage: _localStorage }), true);
    const instance = renderer.getMountedInstance();
    instance.setState({
      'disable-cookie': true,
      'disable-auto-place': null
    });
    instance.handleSave();

    assert.equal(_localStorage.setItem.callCount, 1);
    assert.equal(_localStorage.removeItem.callCount, 2);
  });
});